"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.options = exports.extensions = void 0;
var postcss_1 = __importDefault(require("postcss"));
var postcss_value_parser_1 = __importDefault(require("postcss-value-parser"));
var polaris_tokens_1 = require("@shopify/polaris-tokens");
var sass_1 = require("../../utilities/sass");
var types_1 = require("../../utilities/types");
var constants_1 = require("../../utilities/constants");
exports.extensions = constants_1.scss.extensions;
exports.options = __assign({}, constants_1.scss.options);
function transformer(file, _, options) {
    return (0, postcss_1["default"])(plugin(options)).process(file.source, {
        syntax: require('postcss-scss')
    }).css;
}
exports["default"] = transformer;
var processed = Symbol('processed');
var plugin = function (options) {
    if (options === void 0) { options = {}; }
    var namespacedColor = (0, sass_1.namespace)('color', options);
    return {
        postcssPlugin: 'v9-scss-replace-color',
        Declaration: function (decl) {
            // @ts-expect-error - Skip if processed so we don't process it again
            if (decl[processed])
                return;
            if (!(0, types_1.isKeyOf)(propertyMaps, decl.prop))
                return;
            var replacementMap = propertyMaps[decl.prop];
            var parsed = (0, postcss_value_parser_1["default"])(decl.value);
            parsed.walk(function (node) {
                var _a, _b, _c, _d;
                if (node.type !== 'function')
                    return;
                if (node.value === 'rgba') {
                    return sass_1.StopWalkingFunctionNodes;
                }
                // 1. Remove color() fallbacks
                if (node.value === 'var') {
                    var args = (0, sass_1.getFunctionArgs)(node);
                    var polarisCustomPropertyIndex = args.findIndex(function (arg) {
                        return polarisCustomPropertyRegEx.test(arg);
                    });
                    var colorFnFallbackIndex = args.findIndex(function (arg) {
                        return arg.startsWith(namespacedColor);
                    });
                    if (polarisCustomPropertyIndex < colorFnFallbackIndex) {
                        node.nodes = [node.nodes[0]];
                    }
                    return sass_1.StopWalkingFunctionNodes;
                }
                // 2. Replace `color()` with variable
                if (node.value === namespacedColor) {
                    var colorFnArgs = (0, sass_1.getFunctionArgs)(node).map(sass_1.stripQuotes);
                    var hue = (_a = colorFnArgs[0]) !== null && _a !== void 0 ? _a : '';
                    var value = (_b = colorFnArgs[1]) !== null && _b !== void 0 ? _b : 'base';
                    var forBackground = colorFnArgs[2];
                    // Skip color() with for-background argument
                    if (forBackground)
                        return;
                    // Skip if we don't have a color for the hue and value
                    if (!((0, types_1.isKeyOf)(replacementMap, hue) &&
                        (0, types_1.isKeyOf)(replacementMap[hue], value)))
                        return;
                    var colorCustomProperty = replacementMap[hue][value];
                    node.value = 'var';
                    node.nodes = [
                        {
                            type: 'word',
                            value: colorCustomProperty,
                            sourceIndex: (_d = (_c = node.nodes[0]) === null || _c === void 0 ? void 0 : _c.sourceIndex) !== null && _d !== void 0 ? _d : 0,
                            sourceEndIndex: colorCustomProperty.length
                        },
                    ];
                }
            });
            decl.value = parsed.toString();
            // @ts-expect-error - Mark the declaration as processed
            decl[processed] = true;
        }
    };
};
/*
 * See the legacy Sass API file for the original color palette
 * documentation/guides/legacy-polaris-v8-public-api.scss
 */
var colorMap = {
    blue: {
        dark: '--p-interactive-hovered',
        base: '--p-interactive'
    },
    green: {
        dark: '--p-text-success',
        base: '--p-text-success'
    },
    yellow: {
        dark: '--p-text-warning',
        base: '--p-text-warning'
    },
    red: {
        dark: '--p-text-critical',
        base: '--p-text-critical'
    },
    ink: {
        base: '--p-text',
        light: '--p-text-subdued',
        lighter: '--p-text-subdued',
        lightest: '--p-text-subdued'
    },
    sky: {
        dark: '--p-text-subdued-on-dark',
        base: '--p-text-on-dark',
        light: '--p-text-on-dark',
        lighter: '--p-text-on-dark'
    },
    black: {
        base: '--p-text'
    },
    white: {
        base: '--p-text-on-dark'
    }
};
var backgroundColorMap = {
    green: {
        light: '--p-surface-success',
        lighter: '--p-surface-success-subdued'
    },
    yellow: {
        light: '--p-surface-warning',
        lighter: '--p-surface-warning-subdued'
    },
    red: {
        light: '--p-surface-critical',
        lighter: '--p-surface-critical-subdued'
    },
    ink: {
        dark: '--p-surface-dark',
        base: '--p-surface-neutral-subdued-dark'
    },
    sky: {
        base: '--p-surface-neutral',
        light: '--p-surface-neutral-subdued',
        lighter: '--p-surface-subdued'
    },
    black: {
        base: '--p-surface-dark'
    },
    white: {
        base: '--p-surface'
    }
};
var borderColorMap = {
    green: {
        dark: '--p-border-success',
        base: '--p-border-success',
        light: '--p-border-success-subdued',
        lighter: '--p-border-success-subdued'
    },
    yellow: {
        dark: '--p-border-warning',
        base: '--p-border-warning',
        light: '--p-border-warning-disabled',
        lighter: '--p-border-warning-subdued'
    },
    red: {
        dark: '--p-border-critical',
        base: '--p-border-critical',
        light: '--p-border-critical-subdued',
        lighter: '--p-border-critical-subdued'
    },
    ink: {
        lightest: '--p-border'
    },
    sky: {
        light: '--p-border-subdued'
    }
};
var fillColorMap = {
    green: {
        dark: '--p-icon-success',
        base: '--p-icon-success'
    },
    yellow: {
        dark: '--p-icon-warning',
        base: '--p-icon-warning'
    },
    red: {
        dark: '--p-icon-critical',
        base: '--p-icon-critical'
    },
    ink: {
        base: '--p-icon',
        light: '--p-icon',
        lighter: '--p-icon-subdued',
        lightest: '--p-icon-disabled'
    },
    black: {
        base: '--p-icon'
    },
    white: {
        base: '--p-icon-on-dark'
    }
};
var propertyMaps = {
    color: colorMap,
    background: backgroundColorMap,
    'background-color': backgroundColorMap,
    border: borderColorMap,
    'border-color': borderColorMap,
    fill: fillColorMap
};
var polarisCustomPropertyRegEx = new RegExp(Object.keys(polaris_tokens_1.colors).map(polaris_tokens_1.createVar).join('|'));
